package errors

import "fmt"

// Error is an error that contains positional information.
type Error interface {
	error
	Position () Position
}

type errInternal struct {
	position Position
	message  string
}
var _ Error = new(errInternal)

func (err *errInternal) Error () string {
	return err.String()
}

func (err *errInternal) String () string {
	return err.message
}

func (err *errInternal) Position () Position {
	return err.position
}

// Errorf produces an error with a fmt'd message.
func Errorf (position Position, format string, variables ...any) Error {
	return &errInternal {
		position: position,
		message:  fmt.Sprintf(format, variables...),
	}
}

// Format returns a formatted string representing an error. This string may take
// up multiple lines and contain ANSI escape codes. If err does not fulfill the
// Error interface, err.Error() is returned instead.
func Format (err error) string {
	if err, ok := err.(Error); ok {
		return fmt.Sprintf (
			"%v: %v\n%v",
			err.Position(),
			err.Error(),
			err.Position().Format())
	} else {
		return err.Error()
	}
}
